@@ -684,6 +684,20 @@ print "\n", <<TERM, "\n";
   byte order: $order
 TERM
 
+print dots("Checking perl5 modules");
+eval { require ExtUtils::Command };
+if ($@ ne '') {
+    softfail("Missing ExtUtils::Command!\n" .
+    "    without the perl5 module ExtUtils::Command, 'make install' will fail.\n" .
+    "    In addition, since ExtUtil::Command is normally included in a\n" .
+    "    perl5 installation, you may have an incomplete one that also\n" .
+    "    misses some additional modules that are used in the configure\n" .
+    "    scripts of nqp and rakudo.");
+}
+else {
+    print "OK\n";
+}
+
 print dots('Configuring 3rdparty libs');
 
 my @thirdpartylibs;
@@ -795,6 +809,26 @@ if ($args{'enable-jit'}) {
     print("You can use --no-jit to build without jit.");
 }
 
+if (!$failed && $args{asan}) {
+    print "\n", <<ASANTERM;
+
+    !!  Attention  !!  Attention  !!
+
+You have built moarvm with ASAN (address sanitizer) turned on.
+In order to successfully build nqp and rakudo, you need to set this environment
+variable, since moarvm doesn't fully clean up on shutdown by default.
+
+    export ASAN_OPTIONS=detect_leaks=0
+
+For more information about available asan options, you can run moar with
+
+    export ASAN_OPTIONS=help=1
+
+in your environment.
+
+ASANTERM
+}
+
 print "\n", $failed ? <<TERM1 : <<TERM2;
 Configuration FAIL. You can try to salvage the generated Makefile.
 TERM1
