@@ -116,14 +116,15 @@ static void dump_bb(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g, MVMSpes
     cur_ins = bb->first_ins;
     while (cur_ins) {
         MVMSpeshAnn *ann = cur_ins->annotations;
-        MVMuint32 line_number;
+        MVMuint32 line_number = -1;
         MVMuint32 pop_inlines = 0;
         MVMuint32 num_comments = 0;
 
         while (ann) {
-            /* These four annotations carry a deopt index that we can find a
+            /* These annotations carry a deopt index that we can find a
              * corresponding line number for */
             if (ann->type == MVM_SPESH_ANN_DEOPT_ONE_INS
+                || ann->type == MVM_SPESH_ANN_DEOPT_PRE_INS
                 || ann->type == MVM_SPESH_ANN_DEOPT_ALL_INS
                 || ann->type == MVM_SPESH_ANN_DEOPT_INLINE
                 || ann->type == MVM_SPESH_ANN_DEOPT_OSR) {
@@ -149,7 +150,11 @@ static void dump_bb(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g, MVMSpes
                         ann->data.frame_handler_index);
                     break;
                 case MVM_SPESH_ANN_DEOPT_ONE_INS:
-                    appendf(ds, "      [Annotation: INS Deopt One (idx %d -> pc %d; line %d)]\n",
+                    appendf(ds, "      [Annotation: INS Deopt One After Instruction (idx %d -> pc %d; line %d)]\n",
+                        ann->data.deopt_idx, g->deopt_addrs[2 * ann->data.deopt_idx], line_number);
+                    break;
+                case MVM_SPESH_ANN_DEOPT_PRE_INS:
+                    appendf(ds, "      [Annotation: INS Deopt One Before Instruction (idx %d -> pc %d; line %d)]\n",
                         ann->data.deopt_idx, g->deopt_addrs[2 * ann->data.deopt_idx], line_number);
                     break;
                 case MVM_SPESH_ANN_DEOPT_ALL_INS:
@@ -199,9 +204,26 @@ static void dump_bb(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g, MVMSpes
                     appendf(ds, "      [Annotation: INS Deopt Synth (idx %d)]\n",
                         ann->data.deopt_idx);
                     break;
+                case MVM_SPESH_ANN_CACHED:
+                    appendf(ds, "      [Annotation: Cached (bytecode offset %d)]\n",
+                        ann->data.bytecode_offset);
+                    break;
                 case MVM_SPESH_ANN_COMMENT:
                     num_comments++;
                     break;
+                case MVM_SPESH_ANN_DELAYED_TEMPS:
+                    appendf(ds, "      [Delayed temps to release: ");
+                    MVMSpeshOperand *ptr = ann->data.temps_to_release;
+                    MVMuint16 insert_comma = 0;
+                    while (ptr->lit_i64 != -1) {
+                        if (insert_comma++) {
+                            append(ds, ", ");
+                        }
+                        appendf(ds, "%d(%d)", ptr->reg.orig, ptr->reg.i);
+                        ptr++;
+                    }
+                    appendf(ds, ")]\n");
+                    break;
                 default:
                     appendf(ds, "      [Annotation: %d (unknown)]\n", ann->type);
             }
@@ -214,7 +236,7 @@ static void dump_bb(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g, MVMSpes
             ann = cur_ins->annotations;
             while (ann) {
                 if (ann->type == MVM_SPESH_ANN_COMMENT) {
-                    appendf(ds, "      # %s\n", ann->data.comment);
+                    appendf(ds, "      # [%03d] %s\n", ann->order, ann->data.comment);
                 }
                 ann = ann->next;
             }
@@ -260,7 +282,7 @@ static void dump_bb(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g, MVMSpes
                                 ascension++, cursor = &cursor->outer->body) { };
                         if (cursor->fully_deserialized) {
                             if (cur_ins->operands[i].lex.idx < cursor->num_lexicals) {
-                                char *cstr = MVM_string_utf8_encode_C_string(tc, cursor->lexical_names_list[cur_ins->operands[i].lex.idx]->key);
+                                char *cstr = MVM_string_utf8_encode_C_string(tc, cursor->lexical_names_list[cur_ins->operands[i].lex.idx]);
                                 appendf(ds, ",%s)", cstr);
                                 MVM_free(cstr);
                             } else {
@@ -291,6 +313,10 @@ static void dump_bb(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g, MVMSpes
                             appendf(ds, "liti16(%"PRId16")", cur_ins->operands[i].lit_i16);
                             size += 2;
                             break;
+                        case MVM_operand_uint16:
+                            appendf(ds, "litui16(%"PRIu16")", cur_ins->operands[i].lit_ui16);
+                            size += 2;
+                            break;
                         case MVM_operand_int32:
                             appendf(ds, "liti32(%"PRId32")", cur_ins->operands[i].lit_i32);
                             size += 4;
@@ -303,6 +329,10 @@ static void dump_bb(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g, MVMSpes
                             appendf(ds, "liti64(%"PRId64")", cur_ins->operands[i].lit_i64);
                             size += 8;
                             break;
+                        case MVM_operand_uint64:
+                            appendf(ds, "liti64(%"PRIu64")", cur_ins->operands[i].lit_ui64);
+                            size += 8;
+                            break;
                         case MVM_operand_num32:
                             appendf(ds, "litn32(%f)", cur_ins->operands[i].lit_n32);
                             size += 4;
@@ -323,7 +353,7 @@ static void dump_bb(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g, MVMSpes
                             MVMCallsite *callsite = g->sf->body.cu->body.callsites[cur_ins->operands[i].callsite_idx];
                             appendf(ds, "callsite(%p, %d arg, %d pos, %s, %s)",
                                     callsite,
-                                    callsite->arg_count, callsite->num_pos,
+                                    callsite->flag_count, callsite->num_pos,
                                     callsite->has_flattening ? "flattening" : "nonflattening",
                                     callsite->is_interned ? "interned" : "noninterned");
                             size += 2;
@@ -380,7 +410,7 @@ static void dump_bb(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g, MVMSpes
                 if (sc)
                     result = (MVMCollectable *)MVM_sc_try_get_object(tc, sc, idx);
                 if (result) {
-                    if (result->flags & MVM_CF_STABLE) {
+                    if (result->flags1 & MVM_CF_STABLE) {
                         debug_name = MVM_6model_get_stable_debug_name(tc, (MVMSTable *)result);
                         repr_name  = ((MVMSTable *)result)->REPR->name;
                     } else {
@@ -401,7 +431,7 @@ static void dump_bb(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g, MVMSpes
             ann = cur_ins->annotations;
             while (ann) {
                 if (ann->type == MVM_SPESH_ANN_COMMENT) {
-                    appendf(ds, "  # %s", ann->data.comment);
+                    appendf(ds, "  # [%03d] %s", ann->order, ann->data.comment);
                     break;
                 }
                 ann = ann->next;
@@ -514,6 +544,9 @@ static void dump_facts(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g) {
                 if (g->facts[i][j].writer && g->facts[i][j].writer->info->opcode == MVM_SSA_PHI) {
                     appendf(ds, " (merged from %d regs)", g->facts[i][j].writer->info->num_operands - 1);
                 }
+                if (flags & 1) {
+                    appendf(ds, " (type: %s)", MVM_6model_get_debug_name(tc, g->facts[i][j].type));
+                }
             }
             else {
                 appendf(ds, "    r%d(%d): usages=%d%s", i, j,
@@ -528,33 +561,43 @@ static void dump_facts(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g) {
     }
 }
 
-static void dump_callsite(MVMThreadContext *tc, DumpStr *ds, MVMCallsite *cs) {
+static void dump_callsite(MVMThreadContext *tc, DumpStr *ds, MVMCallsite *cs, char *indent) {
     MVMuint16 i;
-    appendf(ds, "Callsite %p (%d args, %d pos)\n", cs, cs->arg_count, cs->num_pos);
-    for (i = 0; i < (cs->arg_count - cs->num_pos) / 2; i++) {
-        if (cs->arg_names[i]) {
-            char * argname_utf8 = MVM_string_utf8_encode_C_string(tc, cs->arg_names[i]);
-            appendf(ds, "  - %s\n", argname_utf8);
-            MVM_free(argname_utf8);
-        }
+    appendf(ds, "Callsite %p (%d args, %d pos)\n", cs, cs->flag_count, cs->num_pos);
+    for (i = 0; i < cs->flag_count - cs->num_pos; i++) {
+        char * argname_utf8 = MVM_string_utf8_encode_C_string(tc, cs->arg_names[i]);
+        appendf(ds, "%s  - %s\n", indent, argname_utf8);
+        MVM_free(argname_utf8);
     }
     if (cs->num_pos)
-        append(ds, "Positional flags: ");
+        appendf(ds, "%sPositional flags: ", indent);
     for (i = 0; i < cs->num_pos; i++) {
         MVMCallsiteEntry arg_flag = cs->arg_flags[i];
+        MVMCallsiteEntry arg_type = arg_flag & MVM_CALLSITE_ARG_TYPE_MASK;
+        MVMCallsiteEntry other_flags = arg_flag & ~MVM_CALLSITE_ARG_TYPE_MASK;
 
         if (i)
             append(ds, ", ");
 
-        if (arg_flag == MVM_CALLSITE_ARG_OBJ) {
+        if (arg_type == MVM_CALLSITE_ARG_OBJ) {
             append(ds, "obj");
-        } else if (arg_flag == MVM_CALLSITE_ARG_INT) {
+        } else if (arg_type == MVM_CALLSITE_ARG_INT) {
             append(ds, "int");
-        } else if (arg_flag == MVM_CALLSITE_ARG_NUM) {
+        } else if (arg_type == MVM_CALLSITE_ARG_UINT) {
+            append(ds, "uint");
+        } else if (arg_type == MVM_CALLSITE_ARG_NUM) {
             append(ds, "num");
-        } else if (arg_flag == MVM_CALLSITE_ARG_STR) {
+        } else if (arg_type == MVM_CALLSITE_ARG_STR) {
             append(ds, "str");
         }
+        if (other_flags) {
+            if (other_flags == MVM_CALLSITE_ARG_LITERAL) {
+                append(ds, "lit");
+            }
+            else {
+                appendf(ds, "??%d", arg_flag);
+            }
+        }
     }
     if (cs->num_pos)
         append(ds, "\n");
@@ -564,7 +607,7 @@ static void dump_callsite(MVMThreadContext *tc, DumpStr *ds, MVMCallsite *cs) {
 static void dump_fileinfo(MVMThreadContext *tc, DumpStr *ds, MVMStaticFrame *sf) {
     MVMBytecodeAnnotation *ann = MVM_bytecode_resolve_annotation(tc, &sf->body, 0);
     MVMCompUnit            *cu = sf->body.cu;
-    MVMint32           str_idx = ann ? ann->filename_string_heap_index : 0;
+    MVMuint32          str_idx = ann ? ann->filename_string_heap_index : 0;
     MVMint32           line_nr = ann ? ann->line_number : 1;
     MVMString        *filename = cu->body.filename;
     char        *filename_utf8 = "<unknown>";
@@ -579,6 +622,76 @@ static void dump_fileinfo(MVMThreadContext *tc, DumpStr *ds, MVMStaticFrame *sf)
     MVM_free(ann);
 }
 
+static void dump_deopt_pea(MVMThreadContext *tc, DumpStr *ds, MVMSpeshGraph *g) {
+    MVMuint32 i, j;
+    if (MVM_VECTOR_ELEMS(g->deopt_pea.materialize_info)) {
+        append(ds, "\nMaterializations:\n");
+        for (i = 0; i < MVM_VECTOR_ELEMS(g->deopt_pea.materialize_info); i++) {
+            MVMSpeshPEAMaterializeInfo *mat = &(g->deopt_pea.materialize_info[i]);
+            MVMSTable *st = (MVMSTable *)g->spesh_slots[mat->stable_sslot];
+            appendf(ds, "  %d: %s from regs ", i, st->debug_name);
+            for (j = 0; j < mat->num_attr_regs; j++)
+                appendf(ds, j > 0 ? ", r%hu" : "r%hu", mat->attr_regs[j]);
+            append(ds, "\n");
+        }
+    }
+    if (MVM_VECTOR_ELEMS(g->deopt_pea.deopt_point)) {
+        append(ds, "\nDeopt point materialization mappings:\n");
+        for (i = 0; i < MVM_VECTOR_ELEMS(g->deopt_pea.deopt_point); i++) {
+            MVMSpeshPEADeoptPoint *dp = &(g->deopt_pea.deopt_point[i]);
+            appendf(ds, "  At %d materialize %d into r%d\n", dp->deopt_point_idx,
+                    dp->materialize_info_idx, dp->target_reg);
+        }
+    }
+}
+
+void dump_spesh_slots(MVMThreadContext *tc, DumpStr *ds,
+                      MVMuint32 num_spesh_slots, MVMCollectable **spesh_slots,
+                      MVMuint8 show_pointers) {
+    MVMuint32 i;
+    append(ds, "\nSpesh slots:\n");
+    for (i = 0; i < num_spesh_slots; i++) {
+        MVMCollectable *value = spesh_slots[i];
+        appendf(ds, "    %d =", i);
+        if (value == NULL) {
+            append(ds, " NULL\n");
+            continue;
+        }
+
+        if (show_pointers)
+            appendf(ds, " %p", value);
+
+        if (value->flags1 & MVM_CF_STABLE)
+            appendf(ds, " STable (%s)\n",
+                MVM_6model_get_stable_debug_name(tc, (MVMSTable *)value));
+        else if (value->flags1 & MVM_CF_TYPE_OBJECT)
+            appendf(ds, " Type Object (%s)\n",
+                MVM_6model_get_debug_name(tc, (MVMObject *)value));
+        else {
+            MVMObject *obj = (MVMObject *)value;
+            MVMuint32 repr_id = REPR(obj)->ID;
+            appendf(ds, " Instance (%s)",
+                MVM_6model_get_debug_name(tc, obj));
+            if (repr_id == MVM_REPR_ID_MVMStaticFrame || repr_id == MVM_REPR_ID_MVMCode) {
+                MVMStaticFrameBody *body;
+                char *name_str;
+                char *cuuid_str;
+                if (repr_id == MVM_REPR_ID_MVMCode) {
+                    MVMCodeBody *code_body = (MVMCodeBody *)OBJECT_BODY(obj);
+                    obj = (MVMObject *)code_body->sf;
+                }
+                body = (MVMStaticFrameBody *)OBJECT_BODY(obj);
+                name_str  = MVM_string_utf8_encode_C_string(tc, body->name);
+                cuuid_str = MVM_string_utf8_encode_C_string(tc, body->cuuid);
+                appendf(ds, " - '%s' (%s)", name_str, cuuid_str);
+                MVM_free(name_str);
+                MVM_free(cuuid_str);
+            }
+            appendf(ds, "\n");
+        }
+    }
+}
+
 /* Dump a spesh graph into string form, for debugging purposes. */
 char * MVM_spesh_dump(MVMThreadContext *tc, MVMSpeshGraph *g) {
     MVMSpeshBB *cur_bb;
@@ -604,7 +717,7 @@ char * MVM_spesh_dump(MVMThreadContext *tc, MVMSpeshGraph *g) {
     dump_fileinfo(tc, &ds, g->sf);
     append(&ds, ")\n");
     if (g->cs)
-        dump_callsite(tc, &ds, g->cs);
+        dump_callsite(tc, &ds, g->cs, "");
     if (!g->cs)
         append(&ds, "\n");
 
@@ -623,43 +736,12 @@ char * MVM_spesh_dump(MVMThreadContext *tc, MVMSpeshGraph *g) {
 
     /* Dump spesh slots. */
     if (g->num_spesh_slots) {
-        MVMuint32 i;
-        append(&ds, "\nSpesh slots:\n");
-        for (i = 0; i < g->num_spesh_slots; i++) {
-            MVMCollectable *value = g->spesh_slots[i];
-            if (value == NULL)
-                appendf(&ds, "    %d = NULL\n", i);
-            else if (value->flags & MVM_CF_STABLE)
-                appendf(&ds, "    %d = STable (%s)\n", i,
-                    MVM_6model_get_stable_debug_name(tc, (MVMSTable *)value));
-            else if (value->flags & MVM_CF_TYPE_OBJECT)
-                appendf(&ds, "    %d = Type Object (%s)\n", i,
-                    MVM_6model_get_debug_name(tc, (MVMObject *)value));
-            else {
-                MVMObject *obj = (MVMObject *)value;
-                MVMuint32 repr_id = REPR(obj)->ID;
-                appendf(&ds, "    %d = Instance (%s)", i,
-                    MVM_6model_get_debug_name(tc, obj));
-                if (repr_id == MVM_REPR_ID_MVMStaticFrame || repr_id == MVM_REPR_ID_MVMCode) {
-                    MVMStaticFrameBody *body;
-                    char *name_str;
-                    char *cuuid_str;
-                    if (repr_id == MVM_REPR_ID_MVMCode) {
-                        MVMCodeBody *code_body = (MVMCodeBody *)OBJECT_BODY(obj);
-                        obj = (MVMObject *)code_body->sf;
-                    }
-                    body = (MVMStaticFrameBody *)OBJECT_BODY(obj);
-                    name_str  = MVM_string_utf8_encode_C_string(tc, body->name);
-                    cuuid_str = MVM_string_utf8_encode_C_string(tc, body->cuuid);
-                    appendf(&ds, " - '%s' (%s)", name_str, cuuid_str);
-                    MVM_free(name_str);
-                    MVM_free(cuuid_str);
-                }
-                appendf(&ds, "\n");
-            }
-        }
+        dump_spesh_slots(tc, &ds, g->num_spesh_slots, g->spesh_slots, 0);
     }
 
+    /* Dump materialization deopt into. */
+    dump_deopt_pea(tc, &ds, g);
+
     append(&ds, "\n");
 
     /* Print out frame size */
@@ -673,8 +755,8 @@ char * MVM_spesh_dump(MVMThreadContext *tc, MVMSpeshGraph *g) {
 }
 
 /* Dumps a spesh stats type typle. */
-void dump_stats_type_tuple(MVMThreadContext *tc, DumpStr *ds, MVMCallsite *cs,
-                           MVMSpeshStatsType *type_tuple, char *prefix) {
+static void dump_stats_type_tuple(MVMThreadContext *tc, DumpStr *ds, MVMCallsite *cs,
+                                  MVMSpeshStatsType *type_tuple, char *prefix) {
     MVMuint32 j;
     for (j = 0; j < cs->flag_count; j++) {
         MVMObject *type = type_tuple[j].type;
@@ -695,11 +777,11 @@ void dump_stats_type_tuple(MVMThreadContext *tc, DumpStr *ds, MVMCallsite *cs,
 }
 
 /* Dumps the statistics associated with a particular callsite object. */
-void dump_stats_by_callsite(MVMThreadContext *tc, DumpStr *ds, MVMSpeshStatsByCallsite *css) {
+static void dump_stats_by_callsite(MVMThreadContext *tc, DumpStr *ds, MVMSpeshStatsByCallsite *css) {
     MVMuint32 i, j, k;
 
     if (css->cs)
-        dump_callsite(tc, ds, css->cs);
+        dump_callsite(tc, ds, css->cs, "");
     else
         append(ds, "No interned callsite\n");
     appendf(ds, "    Callsite hits: %d\n\n", css->hits);
@@ -729,12 +811,11 @@ void dump_stats_by_callsite(MVMThreadContext *tc, DumpStr *ds, MVMSpeshStatsByCa
                     char *body_name = MVM_string_utf8_encode_C_string(tc, oss->invokes[k].sf->body.name);
                     char *body_cuuid = MVM_string_utf8_encode_C_string(tc, oss->invokes[k].sf->body.cuuid);
                     appendf(ds,
-                        "                %d x static frame '%s' (%s) (caller is outer: %d, multi %d)\n",
+                        "                %d x static frame '%s' (%s) (caller is outer: %d)\n",
                         oss->invokes[k].count,
                         body_name,
                         body_cuuid,
-                        oss->invokes[k].caller_is_outer_count,
-                        oss->invokes[k].was_multi_count);
+                        oss->invokes[k].caller_is_outer_count);
                     MVM_free(body_name);
                     MVM_free(body_cuuid);
                 }
@@ -745,10 +826,10 @@ void dump_stats_by_callsite(MVMThreadContext *tc, DumpStr *ds, MVMSpeshStatsByCa
                         oss->type_tuples[k].arg_types,
                         "                    ");
                 }
-                for (k = 0; k < oss->num_plugin_guards; k++)
-                    appendf(ds, "                %d x spesh plugin guard index %d\n",
-                        oss->plugin_guards[k].count,
-                        oss->plugin_guards[k].guard_index);
+                for (k = 0; k < oss->num_dispatch_results; k++)
+                    appendf(ds, "                %d x dispatch result index %d\n",
+                        oss->dispatch_results[k].count,
+                        oss->dispatch_results[k].result_index);
             }
         }
         append(ds, "\n");
@@ -784,15 +865,6 @@ char * MVM_spesh_dump_stats(MVMThreadContext *tc, MVMStaticFrame *sf) {
 
         for (i = 0; i < ss->num_by_callsite; i++)
             dump_stats_by_callsite(tc, &ds, &(ss->by_callsite[i]));
-
-        if (ss->num_static_values) {
-            append(&ds, "Static values:\n");
-            for (i = 0; i < ss->num_static_values; i++)
-                appendf(&ds, "    - %s (%p) @ %d\n",
-                    MVM_6model_get_stable_debug_name(tc, ss->static_values[i].value->st),
-                    ss->static_values[i].value,
-                    ss->static_values[i].bytecode_offset);
-        }
     }
     else {
         append(&ds, "No spesh stats for this static frame\n");
@@ -833,7 +905,7 @@ char * MVM_spesh_dump_planned(MVMThreadContext *tc, MVMSpeshPlanned *p) {
     /* Dump the callsite of the specialization. */
     if (p->cs_stats->cs) {
         append(&ds, "The specialization is for the callsite:\n");
-        dump_callsite(tc, &ds, p->cs_stats->cs);
+        dump_callsite(tc, &ds, p->cs_stats->cs, "");
     }
     else {
         append(&ds, "The specialization is for when there is no interned callsite.\n");
@@ -875,8 +947,12 @@ char * MVM_spesh_dump_planned(MVMThreadContext *tc, MVMSpeshPlanned *p) {
                 append(&ds, "For unknown reasons.\n");
             break;
         }
-        case MVM_SPESH_PLANNED_DERIVED_TYPES:
+        case MVM_SPESH_PLANNED_DERIVED_TYPES: {
+            MVMCallsite *cs = p->cs_stats->cs;
+            append(&ds, "It was planned for the type tuple:\n");
+            dump_stats_type_tuple(tc, &ds, cs, p->type_tuple, "    ");
             break;
+        }
     }
 
     appendf(&ds, "\nThe maximum stack depth is %d.\n\n", p->max_depth);
@@ -884,23 +960,139 @@ char * MVM_spesh_dump_planned(MVMThreadContext *tc, MVMSpeshPlanned *p) {
     return ds.buffer;
 }
 
-/* Dumps a static frame's guard set into a string. */
-char * MVM_spesh_dump_arg_guard(MVMThreadContext *tc, MVMStaticFrame *sf) {
-    MVMSpeshArgGuard *ag = sf->body.spesh->body.spesh_arg_guard;
+static const char *register_type(MVMint8 reg_type) {
+    switch (reg_type) {
+    case MVM_reg_int8: return "int8";
+    case MVM_reg_int16: return "int16";
+    case MVM_reg_int32: return "int32";
+    case MVM_reg_int64: return "int64";
+    case MVM_reg_num32: return "num32";
+    case MVM_reg_num64: return "num64";
+    case MVM_reg_str: return "str";
+    case MVM_reg_obj: return "obj";
+    case MVM_reg_uint8: return "uint8";
+    case MVM_reg_uint16: return "uint16";
+    case MVM_reg_uint32: return "uint32";
+    case MVM_reg_uint64: return "uint64";
+    default: return "unknown";
+    }
+}
 
+#ifdef DEBUG_HELPERS
+char * MVM_spesh_dump_register_layout(MVMThreadContext *tc, MVMFrame *f) {
+    DumpStr ds;
+    ds.alloc  = 8192;
+    ds.buffer = MVM_malloc(ds.alloc);
+    ds.pos    = 0;
+
+    MVMSpeshCandidate *cand = f->spesh_cand;
+    MVMuint16 num_locals = cand ? cand->body.num_locals : f->static_info->body.work_size / sizeof(MVMRegister);
+    MVMuint16 *local_types = cand ? cand->body.local_types : f->static_info->body.local_types;
+
+    MVMuint16 num_inlines = cand ? cand->body.num_inlines : 0;
+
+    if (num_inlines) {
+        append(&ds, "Inlines:\n");
+    }
+    for (MVMuint16 inl_idx = 0; inl_idx < num_inlines; inl_idx++) {
+        MVMSpeshInline *inl = &cand->body.inlines[inl_idx];
+
+        appendf(&ds, "  - %2u '", inl_idx);
+        append_str(tc, &ds, inl->sf->body.name);
+        append(&ds, "' (cuuid ");
+        append_str(tc, &ds, inl->sf->body.cuuid);
+        append(&ds, ")\n    ");
+        appendf(&ds, "    bytecode from % 4d to % 4d\n    ", inl->start, inl->end);
+        dump_callsite(tc, &ds, inl->cs, "    ");
+    }
+
+    if (f->params.arg_info.callsite->flag_count > 0) {
+        append(&ds, "Parameters:\n");
+        appendf(&ds, "  source: f->");
+        MVMFrame *fp = f;
+
+        while (fp != NULL) {
+            if (fp->work == f->params.arg_info.source) {
+                appendf(&ds, "work: (MVMRegister *)%p\n", f->params.arg_info.source);
+                appendf(&ds, "  frame of source: (MVMFrame *)%p\n", fp);
+                break;
+            }
+            fp = fp->caller;
+            append(&ds, "caller->");
+        }
+        if (fp == NULL) {
+            append(&ds, "...??? not found - flattening involved?\n");
+        }
+
+        append(&ds, "  callsite of params: ");
+        dump_callsite(tc, &ds, f->params.arg_info.callsite, "    ");
+        append(&ds, "\n");
+    }
+
+    append(&ds, "Locals (registers)\n");
+
+    for (MVMuint16 loc_idx = 0; loc_idx < num_locals; loc_idx++) {
+        MVMuint16 type = local_types[loc_idx];
+
+        appendf(&ds, "  %3u: (%7s) ", loc_idx, register_type(type));
+
+        if (type == MVM_reg_obj) {
+            MVMObject *ov = f->work[loc_idx].o;
+            if (ov)
+                appendf(&ds, "%p (%s of %s name %s) ", ov, IS_CONCRETE(ov) ? "conc" : "type", REPR(ov)->name, MVM_6model_get_debug_name(tc, ov));
+            else
+                appendf(&ds, "%p ", ov);
+        }
+        else if (type == MVM_reg_str) {
+            appendf(&ds, "%p ", f->work[loc_idx].s);
+        }
+        else if (type == MVM_reg_int64) {
+            appendf(&ds, "0x%lx ", f->work[loc_idx].i64);
+        }
+
+        for (MVMuint16 inl_idx = 0; inl_idx < num_inlines; inl_idx++) {
+            MVMSpeshInline *inl = &cand->body.inlines[inl_idx];
+
+            MVMuint8 is_start = inl->locals_start == loc_idx ? 1 : 0;
+            MVMuint8 is_coderef = inl->code_ref_reg == loc_idx ? 1 : 0;
+            MVMuint8 is_res = inl->res_reg == loc_idx ? 1 : 0;
+            MVMuint8 is_end = inl->locals_start + inl->num_locals - 1 == loc_idx ? 1 : 0;
+
+            if (is_start) { appendf(&ds, " [start of inline %d's registers]", inl_idx); }
+            if (is_coderef) { appendf(&ds, " [inline %d's code ref register]", inl_idx); }
+            if (is_res) { appendf(&ds, " [inline %d's result register]", inl_idx); }
+            if (is_end) { appendf(&ds, " [last of %d's registers]", inl_idx); }
+        }
+
+        append(&ds, "\n");
+    }
+
+    if (cand) {
+        dump_spesh_slots(tc, &ds, cand->body.num_spesh_slots, f->effective_spesh_slots, 1);
+    }
+
+    append_null(&ds);
+    return ds.buffer;
+}
+#endif
+
+/* Dumps a static frame's guard set into a string. */
+char * MVM_spesh_dump_arg_guard(MVMThreadContext *tc, MVMStaticFrame *sf, MVMSpeshArgGuard *ag) {
     DumpStr ds;
     ds.alloc  = 8192;
     ds.buffer = MVM_malloc(ds.alloc);
     ds.pos    = 0;
 
     /* Dump name and CUID. */
-    append(&ds, "Latest guard tree for '");
-    append_str(tc, &ds, sf->body.name);
-    append(&ds, "' (cuid: ");
-    append_str(tc, &ds, sf->body.cuuid);
-    append(&ds, ", file: ");
-    dump_fileinfo(tc, &ds, sf);
-    append(&ds, ")\n\n");
+    if (sf) {
+        append(&ds, "Latest guard tree for '");
+        append_str(tc, &ds, sf->body.name);
+        append(&ds, "' (cuid: ");
+        append_str(tc, &ds, sf->body.cuuid);
+        append(&ds, ", file: ");
+        dump_fileinfo(tc, &ds, sf);
+        append(&ds, ")\n\n");
+    }
 
     /* Dump nodes. */
     if (ag) {
@@ -932,9 +1124,6 @@ char * MVM_spesh_dump_arg_guard(MVMThreadContext *tc, MVMStaticFrame *sf) {
                     appendf(&ds, "%u: DEREF_RW %u | Y: %u, N: %u\n",
                         i, agn->offset, agn->yes, agn->no);
                     break;
-                case MVM_SPESH_GUARD_OP_CERTAIN_RESULT:
-                    appendf(&ds, "%u: CERTAIN RESULT %u\n", i, agn->result);
-                    break;
                 case MVM_SPESH_GUARD_OP_RESULT:
                     appendf(&ds, "%u: RESULT %u\n", i, agn->result);
                     break;
